import math
import os

class SceneExporter:
	def __init__(self):
		self.external_tech = True
		self.resource_collection = True

	def export(self, context, out_file):
		objs = context.selected_objects
		objs = [o for o in objs if o.type=="MESH" and (not o.compound or o.parent not in objs) and not o.lod_for_parent]

		from .outfile import open_output
		out_file = open_output(out_file)

		path, base = os.path.split(out_file.filename)
		base, ext = os.path.splitext(base)

		from .export_object import ObjectExporter
		object_export = ObjectExporter()
		object_export.compound = True
		object_export.external_tech = self.external_tech

		from .util import Progress
		progress = Progress(context)
		if self.resource_collection:
			res_out = open_output(os.path.join(path, base+"_resources.mdc"))

			# TODO Export techniques as separate items in the collection
			for i, o in enumerate(objs):
				res_out.begin("object", '"{}.object"'.format(o.name))
				progress.push_task(o.name, i/len(objs), (i+1)/len(objs))
				object_export.export(context, res_out, [o], progress)
				progress.pop_task()
				res_out.end()
		else:
			object_export.separate_tech = True
			res_dir = os.path.join(path, base+"_resources")
			if not os.path.exists(res_dir):
				os.makedirs(res_dir)
			for o in objs:
				obj_out = open_output(os.path.join(res_dir, o.name+".object"))
				progress.push_task(o.name, i/len(objs), (i+1)/len(objs))
				object_export.export(context, obj_out, [o], progress)
				progress.pop_task()

		for o in objs:
			out_file.begin("object", '"{}.object"'.format(o.name))
			# XXX Parent relationships screw up the location and rotation
			out_file.write("position", o.location[0], o.location[1], o.location[2])
			if o.rotation_mode=="AXIS_ANGLE":
				angle = o.rotation_axis_angle[0]
				axis = o.rotation_axis_angle[1:]
			else:
				if o.rotation_mode=="QUATERNION":
					q = o.rotation_quaternion
				else:
					q = o.rotation_euler.to_quaternion()
				angle = q.angle
				axis = q.axis
			out_file.write("rotation", angle*180/math.pi, axis[0], axis[1], axis[2])
			out_file.write("scale", o.scale[0], o.scale[1], o.scale[2])
			out_file.end();
